%% Flow Cytometry exercise solution
%   Written by BB 2018/08/07

% It's usually helpful to have scipts begin with these two lines 
% which clear any variables and close any open plots 
clear
close all

% Load data
load facs_data.mat

% Column order is: 'Time','YFP','BFP','mCherry','FSC','SSC'

% Log10 transform the data (not the time column)
facs_data(:,2:end) = log10(facs_data(:,2:end));


%% Histograms of raw data 

% When comparing distributions, it's a good idea to make sure the bins are 
% the same for each histogram. For the hist function, we can specify the 
% centers of each bin. I'll also use 'ylim' to make sure the y-axis is the 
% same for each one.

binCenters = 1.05:0.1:4.95; %This will be 40 bins 

% YFP histogram
figure;
subplot(3,1,1);
hist(facs_data(:,2),binCenters)
title('YFP')
xlabel('Fluorescence')
ylabel('Counts')
ylim([0 2000])

% BFP histogram
subplot(3,1,2);
hist(facs_data(:,3),binCenters)
title('BFP')
xlabel('Fluorescence')
ylabel('Counts')
ylim([0 2000])

% mCherry histogram
subplot(3,1,3);
hist(facs_data(:,4),binCenters)
title('mCherry')
xlabel('Fluorescence')
ylabel('Counts')
ylim([0 2000])


%% Scatter plots 

%First we'll do pairwise comparisons between channels

%YFP vs BFP
figure;
subplot(1,3,1)
plot(facs_data(:,2),facs_data(:,3),'b.');
xlabel('YFP fluorescence');
ylabel('BFP fluorescence');

%YFP vs mCherry
subplot(1,3,2)
plot(facs_data(:,2),facs_data(:,4),'b.');
xlabel('YFP fluorescence');
ylabel('mCherry fluorescence');

%BFP vs mCherry
subplot(1,3,3)
plot(facs_data(:,3),facs_data(:,4),'b.');
xlabel('BFP fluorescence');
ylabel('mCherry fluorescence');


% We can also make a scatter plot versus time. This is helpful to make sure 
% fluorescence is even throughout the run. 

figure; 
hold on
plot(facs_data(:,1),facs_data(:,2),'y.');
plot(facs_data(:,1),facs_data(:,3),'b.');
plot(facs_data(:,1),facs_data(:,4),'r.');
xlabel('Time')
ylabel('Fluorescence')
legend('YFP','BFP','mCherry')

%% Segmentation: Picking a threshold

% For mCherry, the histogram is very biomodal with good separation,
% For BFP, the histogram is less bimodal, so it might help to normalize by
% the cell size.

%Make a new set of binCenters
normBinCenters = 0.01:0.02:1; %50 bins

% BFP/ FSC
figure
subplot(2,2,1)
hist(facs_data(:,3) ./ facs_data(:,5),normBinCenters);
title('BFP/FSC')
xlabel('Normalized fluorescence')
ylabel('Counts')

% BFP/ SSC
subplot(2,2,2)
hist(facs_data(:,3) ./ facs_data(:,6),normBinCenters);
title('BFP/SSC')
xlabel('Normalized fluorescence')
ylabel('Counts')

% mCHerry / FSC
subplot(2,2,3)
hist(facs_data(:,4) ./ facs_data(:,5),normBinCenters);
title('mCherry/FSC')
xlabel('Normalized fluorescence')
ylabel('Counts')

% mCherry / SSC
subplot(2,2,4)
hist(facs_data(:,4) ./ facs_data(:,6),normBinCenters);
title('mCherry/SSC')
xlabel('Normalized fluorescence')
ylabel('Counts')

%Now we can look at the BFP vs. mCherry scatter plot, but with the
%normalized values
figure
subplot(1,2,1)
plot(facs_data(:,3)./ facs_data(:,5),facs_data(:,4)./ facs_data(:,5),'b.');
xlabel('BFP/FSC');
ylabel('mCherry/FSC');

subplot(1,2,2)
plot(facs_data(:,3)./ facs_data(:,6),facs_data(:,4)./ facs_data(:,6),'b.');
xlabel('BFP/SSC');
ylabel('mCherry/SSC');


% While the mCherry values were already very bimodal, we can see from 
% these histograms and scatter plots that normalizing by SSC provides a
% better separation in the BFP channel
%
% We can now identify a threshold that will be used to segment the cells
% Your thresholds can vary, but some appropriate thresholds are:
%
% BFP/SSC: 0.73
% mCherry/SSC: 0.65
%

BFPThresh = 0.73;
mCherryThresh = 0.65;

% Now create to the logical arrays to use for segmenting. I call these
% isBFP and isCherry
isBFP = facs_data(:,3) ./ facs_data(:,6) >= BFPThresh;
ismCherry = facs_data(:,4) ./ facs_data(:,6) >= mCherryThresh;

% We can count the number of labeled cells with the sum command
nBFPCells = sum(isBFP)
nmCherryCells = sum(ismCherry)

%% Segmentation: 

% We could use either isBFP or ismCherry to separate the data into two
% groups. First, we'll check to see how good the segmentation was

% Redo the time scatter plot of BFP vs mCherry but using segmentation
figure
subplot(1,2,1)
hold on
plot(facs_data(isBFP,3),facs_data(isBFP,4),'b.')
plot(facs_data(~isBFP,3),facs_data(~isBFP,4),'k.')
xlabel('BFP fluorescence')
ylabel('mCherry fluorescence')
legend('BFP labeled','Not BFP labeled')

subplot(1,2,2)
hold on
plot(facs_data(ismCherry,3),facs_data(ismCherry,4),'r.')
plot(facs_data(~ismCherry,3),facs_data(~ismCherry,4),'k.')
xlabel('BFP fluorescence')
ylabel('mCherry fluorescence')
legend('mCherry labeled','Not mCherry labeled')

% Notice any overlap in these plots? 

% We can also see this directly with a plot of all four categories
figure
hold on
plot(facs_data(isBFP & ~ismCherry,3),facs_data(isBFP & ~ismCherry,4),'b.')
plot(facs_data(~isBFP & ismCherry,3),facs_data(~isBFP & ismCherry,4),'r.')
plot(facs_data(isBFP & ismCherry,3),facs_data(isBFP & ismCherry,4),'m.')
plot(facs_data(~isBFP & ~ismCherry,3),facs_data(~isBFP & ~ismCherry,4),'k.')
xlabel('BFP fluorescence')
ylabel('mCherry fluorescence')
legend('Only BFP','Only mCherry','Both','Neither','Location','Best')

%% Choose a segmentation
% Which segmentation to use is a difficult question. This particular data 
% set is fairly clean, so either segmentation is okay for now. You could
% even use the combination, i.e. removing cells that are labeled with both
isOnlymCherry = ismCherry & ~isBFP;
isOnlyBFP = isBFP & ~ismCherry;


%% Applying segmentation to YFP values

% Now we look at the YFP expression. We can do several things, like the 
% time scatter plot again, but now segmented
figure; hold on
plot(facs_data(isOnlyBFP,1),facs_data(isOnlyBFP,2),'b.');
plot(facs_data(isOnlymCherry,1),facs_data(isOnlymCherry,2),'r.');
xlabel('Time')
ylabel('YFP Fluorescence')
legend('BFP cells','mCherry cells')

% I can also do histograms of the values. While it's possible to put
% multiple histogram on the same set of axes, I'll just line them up and
% make sure they have the same x and y limits

% BFP Cells
figure; 
subplot(2,1,1)
hist(facs_data(isOnlyBFP,2),binCenters)
xlabel('YFP fluorescence')
ylabel('Counts')
title('BFP cells')
ylim([0 1600])

% mCherry cells
subplot(2,1,2)
hist(facs_data(isOnlymCherry,2),binCenters)
xlabel('YFP fluorescence')
ylabel('Counts')
title('mCherry cells')
ylim([0 1600])

%% Summary statistics

% You can probably see from the histograms that the YFP fluorescence of the 
% BFP cells has a higher mean and standard deviation than the RFP cells. We
% can double check this with a few built-in functions

fprintf('\n--------------YFP fluorescence of BFP cells-----------------\n')
meanBFP = mean(facs_data(isOnlyBFP,2))
medianBFP = median(facs_data(isOnlyBFP,2))
stdBFP = std(facs_data(isOnlyBFP,2))
cvBFP = std(facs_data(isOnlyBFP,2))/mean(facs_data(isOnlyBFP,2))

fprintf('\n-------------YFP fluorescence of mCherry cells--------------\n')
meanMCherry = mean(facs_data(isOnlymCherry,2))
medianMCherry = median(facs_data(isOnlymCherry,2))
stdMCherry = std(facs_data(isOnlymCherry,2))
cvMCherry = std(facs_data(isOnlymCherry,2))/mean(facs_data(isOnlymCherry,2))

